package com.merchantsofrome.server.controller

import com.merchantsofrome.server.model.*
import com.merchantsofrome.server.model.object_parameters.FarmPlotPlanting
import com.merchantsofrome.server.repository.*
import org.joda.time.Instant
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*

import javax.validation.Valid
import kotlin.collections.ArrayList


@RestController
@RequestMapping("/api/treePatches")
class TreePatchController(
        private val treePatchRepository: TreePatchRepository,
        private val accountRepository: AccountRepository,
        private val treeRepository: TreeRepository,
        private val itemRepository: ItemRepository,
        private val itemAccountRepository: ItemAccountRepository,
        private val treeAccountRepository: TreeAccountRepository) {


    @GetMapping
    fun getFarmPlotById(@RequestHeader("Username") username: String): ResponseEntity<List<TreePatch>> {

        val farmPlots: List<TreePatch> = treePatchRepository.getTreePatchesByAccount_Username(username = username)
        return ResponseEntity(farmPlots, HttpStatus.OK)
    }

    //TODO find another solution
    @PostMapping("/create")
    fun createNewTreePatch(@RequestHeader("Username") username: String) {
        val account: Account = accountRepository.findById(username).get()
        for (i in 0..9) {
            val treePatch = TreePatch(slotnr = i, account = account)
            treePatchRepository.save(treePatch)
        }
    }

    @PutMapping("/plant")
    fun plantTreePatchByPlantNameAndSlotnr(@RequestHeader("Username") username: String, @Valid @RequestBody farmPlotPlanting: FarmPlotPlanting): ResponseEntity<Any> {
        val treePatches: ArrayList<TreePatch> = ArrayList()
        val treeOptional = treeRepository.findById(farmPlotPlanting.plant)
        val tree = if(treeOptional.isPresent) treeOptional.get() else return ResponseEntity("No such tree exists", HttpStatus.BAD_REQUEST)
        val fruit = itemRepository.findById(tree.fruitId).get()
        if (!removePlantedFruitsFromInventory(username, tree, farmPlotPlanting.plots.size)) {
            return ResponseEntity("You don't have enough ${fruit.name}s", HttpStatus.CONFLICT)
        }
        farmPlotPlanting.plots.forEach {
            val existingTreePatch: TreePatch = treePatchRepository.getTreePatchBySlotnrAndAccount_Username(it, username)
            if (existingTreePatch.tree != null) {
                return ResponseEntity("Plot $it is not available for planting", HttpStatus.CONFLICT)
            }
            val updatedTreePatch: TreePatch = existingTreePatch
                    .copy(tree = tree, finishedGrowing = Instant.now().millis + tree.growthTimeInMillis, finishedGrowingFruit = Instant.now().millis + tree.growthTimeInMillis + tree.growthTimeFruitInMillis)
            treePatchRepository.save(updatedTreePatch)
            treePatches.add(updatedTreePatch)
        }
        return ResponseEntity(treePatches, HttpStatus.OK)
    }


    @PutMapping("/harvestFruit")
    fun harvestFruitBySlotnr(
            @RequestHeader("Username") username: String,
            @Valid @RequestBody farmPlotPlanting: FarmPlotPlanting): ResponseEntity<Any> {

        val harvest = harvestFruitFromTreePatches(username, farmPlotPlanting)
        return if (harvest != null) {
            addHarvestToInventory(accountRepository.findById(username).get(), harvest)
            ResponseEntity(harvest, HttpStatus.OK)
        } else {
            ResponseEntity("This tree has not finished growing fruit", HttpStatus.CONFLICT)
        }

    }

    @PutMapping("/chopTree")
    fun chopTreePatchBySlotnr(
            @RequestHeader("Username") username: String,
            @Valid @RequestBody farmPlotPlanting: FarmPlotPlanting): ResponseEntity<Any> {

        val harvest = chopTreesFromTreePatches(username, farmPlotPlanting)
        return if (harvest != null) {
            addHarvestToInventory(accountRepository.findById(username).get(), harvest)
            ResponseEntity(harvest, HttpStatus.OK)
        } else {
            ResponseEntity("This tree has not finished growing", HttpStatus.CONFLICT)
        }
    }


    @PostMapping("/buyNewTreePatch")
    fun buyNewTreePatch(@RequestHeader("Username") username: String): ResponseEntity<String> {
        val account: Account = accountRepository.findById(username).get()
        val price: Int = when (account.indexOfLastTreePatchUnlocked) {
            -1 -> 125
            0 -> 500
            1 -> 5_000
            2 -> 50_000
            3 -> 500_000
            else -> return ResponseEntity(HttpStatus.CONFLICT)
        }

        if (account.coinsOwned >= price) {
            val treePatch = TreePatch(slotnr = account.indexOfLastTreePatchUnlocked + 1, account = account)
            treePatchRepository.save(treePatch)
            accountRepository.save(account.copy(indexOfLastTreePatchUnlocked = account.indexOfLastTreePatchUnlocked + 1, coinsOwned = account.coinsOwned - price))
            return ResponseEntity("WP", HttpStatus.OK)
        }
        return ResponseEntity("${account.coinsOwned} price = ${price}", HttpStatus.CONFLICT)
    }

    @PutMapping("/testharvest")
    fun test() {
        val harvest = mutableMapOf<String, Int>()
        harvest["logs"] = 5
        harvest["apple"] = 10
        addHarvestToInventory(accountRepository.findById("kape").get(), harvest)
    }

    fun removePlantedFruitsFromInventory(username: String, tree: Tree, amount: Int): Boolean {
        val fruitId = tree.fruitId
        if(fruitId==-1) return false
        val itemAccount: ItemAccount? = itemAccountRepository.getItemAccountByAccount_UsernameAndItemId(username = username, itemId = fruitId)
        return if (itemAccount != null && itemAccount >= amount) {
            itemAccountRepository.save(itemAccount - amount)
            true
        }else{
            false
        }
    }


    fun harvestFruitFromTreePatches(username: String, farmPlotPlanting: FarmPlotPlanting): MutableMap<String, Int>? {
        val harvest = mutableMapOf<String, Int>().withDefault { 0 }
        farmPlotPlanting.plots.forEach {
            val existingTreePatch: TreePatch? = treePatchRepository.getTreePatchBySlotnrAndAccount_Username(it, username)
            if (existingTreePatch == null || (existingTreePatch.finishedGrowingFruit != null && existingTreePatch.finishedGrowingFruit > Instant.now().millis)) return null
            val tree: Tree? = existingTreePatch.tree
            println(tree)
            if (tree != null) {
                val amount: Int = (tree.amountFruitWhenFullyGrownMin..tree.amountFruitWhenFullyGrownMax).shuffled().last()
                val fruitOptional = itemRepository.findById(tree.fruitId)
                val fruit = if(fruitOptional.isPresent) fruitOptional.get() else return null
                harvest[fruit.name] = harvest.getValue(fruit.name) + amount
                val treeAccount: TreeAccount? = treeAccountRepository.getTreeAccountByAccount_UsernameAndTreeName(username, tree.name)
                if (treeAccount == null) {
                    treeAccountRepository.save(TreeAccount(account = accountRepository.findById(username).get(), tree = tree, numberOfTimesHarvested = 0, totalAmountHarvested = 0, numberOfTimesFruitHarvested = 1, totalAmountFruitHarvested = amount))
                } else {
                    treeAccountRepository.save(treeAccount.copy(numberOfTimesFruitHarvested = treeAccount.numberOfTimesFruitHarvested + 1, totalAmountFruitHarvested = treeAccount.totalAmountFruitHarvested + amount))
                }
                val updatedTreePatch: TreePatch = existingTreePatch
                        .copy(finishedGrowingFruit = Instant.now().millis + tree.growthTimeFruitInMillis)
                treePatchRepository.save(updatedTreePatch)
            }else{
                return null
            }

        }
        return harvest
    }

    fun chopTreesFromTreePatches(username: String, farmPlotPlanting: FarmPlotPlanting): MutableMap<String, Int>? {
        val harvest = harvestFruitFromTreePatches(username, farmPlotPlanting)?: mutableMapOf<String, Int>().withDefault { 0 }
        farmPlotPlanting.plots.forEach {
            val existingTreePatch: TreePatch? = treePatchRepository.getTreePatchBySlotnrAndAccount_Username(it, username)
            if (existingTreePatch == null || (existingTreePatch.finishedGrowing != null && existingTreePatch.finishedGrowing > Instant.now().millis)) return null
            val tree: Tree? = existingTreePatch.tree
            if (tree != null) {
                val amount: Int = (tree.amountLogsWhenFullyGrownMin..tree.amountLogsWhenFullyGrownMax).shuffled().last()
                harvest["logs"] = harvest.getValue("logs") + amount
                val treeAccount: TreeAccount? = treeAccountRepository.getTreeAccountByAccount_UsernameAndTreeName(username, tree.name)
                if (treeAccount == null) {
                    treeAccountRepository.save(TreeAccount(account = accountRepository.findById(username).get(), tree = tree, numberOfTimesHarvested = 1, totalAmountHarvested = amount, numberOfTimesFruitHarvested = 0, totalAmountFruitHarvested = 0))
                } else {
                    treeAccountRepository.save(treeAccount.copy(numberOfTimesHarvested = treeAccount.numberOfTimesHarvested + 1, totalAmountHarvested = treeAccount.totalAmountHarvested + amount))
                }
                val updatedTreePatch: TreePatch = existingTreePatch
                        .copy(tree = null, finishedGrowing = -1, finishedGrowingFruit = -1)
                treePatchRepository.save(updatedTreePatch)
            }else{
                return null
            }
        }
        return harvest
    }

    fun addHarvestToInventory(account: Account, harvest: MutableMap<String, Int>) {
        harvest.forEach { itemname, amount ->
            val item: Item? = itemRepository.getItemByName(itemname)
            if(item != null){
                val itemAccount: ItemAccount? = itemAccountRepository.getItemAccountByAccount_UsernameAndItemId(username = account.username, itemId = item.id)
                if (itemAccount != null) {
                    itemAccountRepository.save(itemAccount + amount)
                } else {
                    val newItemAccount = ItemAccount(account = account, itemId = item.id, amountOwned = amount)
                    itemAccountRepository.save(newItemAccount)
                }
            }
        }
    }
}