import React, {Component} from 'react';
import './Game.css';
import Farm from "./farming/Farm";
import Inventory from "./Inventory";
import {Tab} from "./Enum";
import TabBar from "./TabBar";
import {connect} from "react-redux";
import {Plant} from "./farming/Plant";
import {
    addQuest, answerFeedbackQuestion,
    increaseInventory, loadFeedbackQuestions,
    loadHistory,
    loadInventory,
    loadPrices,
    newMine,
    plantSeeds,
    plantTrees,
    setProcess,
    setQuest,
    setTab,
    setUnlockedSeeds
} from "./actions/actions";
import {CropName} from "./Items/Crops";
import {FeedbackInfo, FeedbackQuestion, Fetch, InventoryList, ItemFactory, PriceList} from "./utils";
import {ReduxState} from "./reducers/reducer";
import {Item} from "./Items/Item";
import QuestView from "./quests/QuestView";
import Mountain from "./mining/Mountain";
import {Mine} from "./mining/Mine";
import SellInventory from "./SellInventory";
import Settings from "./settings/Settings";
import Forest from "./woodcutting/Forest";
import {Tree, TreeName} from "./woodcutting/Tree";
import Div100vh from 'react-div-100vh'
import {Scoreboard} from "./scoreboard/Scoreboard";
import {DialogueStandards} from "./dialogue/DialogueStandards";
import {Items, ItemsName} from "./Items/Items";

interface IGameState {
    key: number,
    questsLoaded: boolean,
    landscape: boolean,
    rulesAccepted: boolean,
}

interface IGameProps {
    currentTab: Tab,
    inventory: InventoryList,
    feedback: FeedbackInfo
    plantSeeds: (plants: Plant[]) => void,
    plantTrees: (trees: Tree[]) => void,
    setUnlockedSeeds: (plants: Plant[]) => void,
    addQuest: (questId: number) => void,
    setTab: (tab: Tab) => void,
    newMines: (mines: Mine[]) => void,
    increaseInventory: (item: Item) => void,
    loadInventory: (inventory: InventoryList) => void,
    setQuest: (questId: number, checkpointReached: number) => void,
    setProcess: (category: string, name: string, time: number, amount: number, finishTime?: Date) => void
    loadPrices: (priceList: PriceList) => void,
    loadHistory: (category: string, history: History) => void,
    loadFeedbackQuestions: (questions: FeedbackQuestion[]) => void,
    answerFeedbackQuestion: () => void

}

const mapStateToProps = (state: ReduxState) => {
    return {
        currentTab: state.currentTab,
        currentQuests: state.currentQuests,
        inventory: state.inventory,
        feedback: state.feedback,
    }
};

function mapDispatchToProps(dispatch: any) {
    return {
        plantSeeds: (plant: Plant[]) => dispatch(plantSeeds(plant)),
        setUnlockedSeeds: (plants: Plant[]) => dispatch(setUnlockedSeeds(plants)),
        addQuest: (questId: number) => dispatch(addQuest(questId)),
        setTab: (tab: Tab) => dispatch(setTab(tab)),
        newMines: (mines: Mine[]) => dispatch(newMine(mines)),
        plantTrees: (trees: Tree[]) => dispatch(plantTrees(trees)),
        increaseInventory: (item: Item) => dispatch(increaseInventory(item)),
        loadInventory: (inventory: InventoryList) => dispatch(loadInventory(inventory)),
        setQuest: (questId: number, checkpointReached: number) => dispatch(setQuest(questId, checkpointReached)),
        setProcess: (category: string, name: string, time: number, amount: number, finishTime?: Date) =>
            dispatch(setProcess(category, name, time, amount, finishTime)),
        loadPrices: (priceList: PriceList) => dispatch(loadPrices(priceList)),
        loadHistory: (category: string, history: History) => dispatch(loadHistory(category,history)),
        loadFeedbackQuestions: (questions: FeedbackQuestion[]) => dispatch(loadFeedbackQuestions(questions)),
        answerFeedbackQuestion: () => dispatch(answerFeedbackQuestion())
    };
}

class ConnectedGame extends Component<IGameProps, IGameState> {
    state = {
        key: Math.random(),
        questsLoaded: false,
        landscape: window.matchMedia('(orientation: landscape)').matches,
        rulesAccepted: true,
    };

    private timeout: any;

    constructor(props: IGameProps) {
        super(props);
        this.setTab = this.setTab.bind(this);
        this.onResize = this.onResize.bind(this);
        this.listener = this.listener.bind(this);
        this.acceptRules = this.acceptRules.bind(this);
    }

    onResize(evt: Event){
        this.setState({
            landscape: window.matchMedia('(orientation: landscape)').matches
        })
    };

    listener(evt: Event){
        clearTimeout(this.timeout);

        this.timeout = setTimeout(() => {
            this.timeout = null;
            this.onResize(evt);
        }, 16);
    }

    componentDidMount() {
        window.addEventListener("resize", this.listener);
        Fetch.get("plants/unlocked")
            .then(response => response.json())
            .then(data => this.props.setUnlockedSeeds(this.verifySeeds(data)))
            .catch(error => console.warn(error));
        Fetch.get("farmPlots")
            .then(response => response.json())
            .then(data => {
                let plants: Plant[] = [];
                for (let key in data) {
                    let obj = data[key];
                    if (obj.plant) {
                        plants.push(new Plant(obj.plant.name as CropName, obj.slotnr, new Date(obj.finishedGrowing)))
                    } else {
                        plants.push(new Plant(CropName.empty, obj.slotnr))
                    }
                }
                this.props.plantSeeds(plants)

            })
            .catch(error => console.warn(error));

        Fetch.get("treePatches")
            .then(response => response.json())
            .then(data => {
                let trees: Tree[] = [];
                for (let key in data) {
                    let obj = data[key];
                    if (obj.tree) {
                        trees.push(new Tree(obj.tree.name as TreeName, obj.slotnr, new Date(obj.finishedGrowing), new Date(obj.finishedGrowingFruit)))
                    } else {
                        trees.push(new Tree(TreeName.empty, obj.slotnr))
                    }
                }
                this.props.plantTrees(trees)
            })
            .catch(error => console.warn(error));
        Fetch.get("inventory")
            .then(response => response.json())
            .then(data => {
                this.props.loadInventory(this.verifyInventory(data))
            })
            .catch(error => console.warn(error));
        Fetch.get("quests")
            .then(response => response.json())
            .then(data => {
                for (let i in data) {
                    this.props.setQuest(data[i].questId, data[i].finished ? -1 : data[i].checkpointReached)
                }
                this.setState({questsLoaded: true})
            })
            .catch(error => console.warn(error));
        Fetch.get("process")
            .then(response => response.json())
            .then(data => {
                for (let key in data) {
                    this.props.setProcess(key, data[key].name, 0, data[key].amount, new Date(data[key].finishTime));
                }
            })
            .catch(error => console.warn(error));
        Fetch.get("items/prices")
            .then(response => response.json())
            .then(data => {
                this.props.loadPrices(data)
            })
            .catch(error => console.warn(error));
        // Henting av gruver
        Fetch.get("mining/getallminesatstart").then(response => {
            switch (response.status) {
                case 200:
                    response.json().then(result => {
                        for (let i = 0; i < result.length; i++) {
                            let mineId = result[i][0];
                            let timeToMine = result[i][1];
                            let finishedTime = result[i][2];
                            this.props.newMines([new Mine(mineId, timeToMine, 3, finishedTime)]);
                        }
                    }).catch(error => {
                        console.error(error);
                    });
                    break;

                default:
                    console.warn(response.status + " " + response.statusText);
                    break;
            }
        }).catch(error => {
            console.error(error);
        });
        Fetch.get("plants/harvested")
            .then(response => response.json())
            .then(history => {
                this.props.loadHistory("plants", history)
            })
            .catch(error => console.warn(error));
        Fetch.get("trees/harvested")
            .then(response => response.json())
            .then(history => {
                this.props.loadHistory("trees", history)
            })
            .catch(error => console.warn(error));
        Fetch.get("feedback/questions")
            .then(response => response.json())
            .then(questions => this.props.loadFeedbackQuestions(questions));
        Fetch.get("accounts/rules")
            .then(response => response.json())
            .then(response => this.setState({rulesAccepted: response.value}))
    }

    componentWillUnmount(): void {
        clearTimeout(this.timeout);
        window.removeEventListener("resize", this.listener)
    }

    verifySeeds(input: any[]) {
        let plants: Plant[] = [];
        for (let key in input) {
            plants.push(new Plant(input[key].name as CropName))
        }
        plants.sort((a,b)=>a.timeToGrow - b.timeToGrow);
        return plants;
    }

    verifyInventory(inventory: InventoryList): InventoryList {
        let newInventory = new InventoryList();
        for (let key in inventory) {
            newInventory[key] = ItemFactory.create(inventory[key].name, inventory[key].amount);
        }
        return newInventory;
    }

    setTab(tab: Tab) {
        if (this.props.currentTab == tab && tab == Tab.quest) {
            this.setState({
                key: Math.random()
            })
        } else {
            this.props.setTab(tab);
        }
    }

    tab() {
        switch (this.props.currentTab) {
            case Tab.farming:
                return <Farm/>;
            case Tab.quest:
                return <QuestView key={this.state.key} loaded={this.state.questsLoaded}/>;
            case Tab.mining:
                return <Mountain/>;
            case Tab.woodcutting:
                return <Forest/>;
            case Tab.sellInventory:
                return <SellInventory/>;
            case Tab.scoreboard:
                return <Scoreboard />;
            case Tab.settings:
                return <Settings acceptRules={this.acceptRules} rulesAccepted={this.state.rulesAccepted}/>;
            case Tab.feedback:
                return this.feedback();
        }
    }

    feedback(){
        if(this.props.feedback.finished){
            return null;
        }
        let currentQuestion = this.props.feedback.questions[this.props.feedback.progress];
        switch(currentQuestion.questionType){
            case "info":
                return DialogueStandards.CONFIRM_DIALOGUE_ELEMENT_BODY(
                    "Feedback",
                    this.splitFeedbackText(currentQuestion.questionText),
                    ()=>{
                        this.props.answerFeedbackQuestion();
                        Fetch.post("feedback/submit",{
                            id: 0,
                            username: Fetch.username,
                            questionId: currentQuestion.id,
                            answer: "confirm"
                        }).catch(error=>console.warn(error))
                    },
                    false
                );
            case "text":
                return DialogueStandards.TEXT_INPUT_DIALOGUE_ELEMENT_BODY(
                    "Feedback",
                    this.splitFeedbackText(currentQuestion.questionText),
                    (value: string)=>{
                        this.props.answerFeedbackQuestion();
                        Fetch.post("feedback/submit",{
                            id: 0,
                            username: Fetch.username,
                            questionId: currentQuestion.id,
                            answer: value
                        }).catch(error=>console.warn(error))
                    },
                    undefined,
                    undefined,
                    255,
                    false
                );
            case "end":
                return DialogueStandards.CONFIRM_DIALOGUE_ELEMENT_BODY(
                    "Feedback",
                    this.splitFeedbackText(currentQuestion.questionText),
                    ()=>{
                        this.props.increaseInventory(new Items(ItemsName.coins, Number(currentQuestion.questionText.split("~")[1].split(" ")[0])));
                        this.props.answerFeedbackQuestion();
                        Fetch.post("feedback/submit",{
                            id: 0,
                            username: Fetch.username,
                            questionId: currentQuestion.id,
                            answer: "confirm"
                        }).catch(error=>console.warn(error))
                    },
                    false
                );
        }
    }

    splitFeedbackText(text: string): JSX.Element{
        return <div>{text.split("~").map((a,i)=><p key={i}>{a}</p>)}</div>;

    }

    acceptRules(){
        this.setState({rulesAccepted: true});
        Fetch.post("accounts/rules")
            .catch(error=>console.warn(error))
    }

    getHeight(){
        return window.matchMedia('(orientation: landscape)').matches? "100rvh" : "calc(100rvh - 70px)";
    }

    showInventory(){
        switch(this.props.currentTab){
            case Tab.sellInventory:
            case Tab.quest:
            case Tab.scoreboard:
                return null;
            default:
                return <Inventory />

        }
    }

    render() {
        return (
            <div className="Game">
                <TabBar setTab={this.setTab} rulesAccepted={this.state.rulesAccepted}/>
                <Div100vh className={"game-tab"} style={{height: this.getHeight()}}>
                    {this.showInventory()}
                    {this.tab()}
                </Div100vh>
            </div>
        )
    }
}

const Game = connect(mapStateToProps, mapDispatchToProps)(ConnectedGame);

export default Game;
