import React, {Component} from "react";
import './SellInventory.css';
import {connect} from "react-redux";
import SubInventory from "./SubInventory";
import {
    DialogueTools,
    Fetch,
    InventoryList,
    InventoryTools,
    ItemFactory,
    PriceList,
    ProcessData,
    Strings
} from "./utils";
import {Item} from "./Items/Item";
import {ItemBox} from "./ItemBox";
import {Items, ItemsName} from "./Items/Items";
import {DialogueStandards} from "./dialogue/DialogueStandards";
import {increaseInventory} from "./actions/actions";
import {ReduxState} from "./reducers/reducer";
import {Category} from "./Enum";

enum Dialogue {
    None,
    Sell,
    Error
}

interface ISellInventoryState {
    dialogue: Dialogue;
    error: string;
    item: Item;
    inputValue: number
}

interface ISellInventoryProps {
    inventory: InventoryList;
    increaseInventory: (item: Item) => void;
    prices: PriceList;
}

const mapStateToProps = (state: ReduxState) => {
    return {
        inventory: state.inventory,
        prices: state.prices
    }
};

function mapDispatchToProps(dispatch: any) {
    return {
        increaseInventory: (item: Item) => dispatch(increaseInventory(item))
    }
}

class ConnectedSellInventory extends Component<ISellInventoryProps, ISellInventoryState> {

    state = {
        dialogue: Dialogue.None,
        error: "Unspecified error",
        item: new Items(ItemsName.empty),
        inputValue: 1
    };

    constructor(props: ISellInventoryProps) {
        super(props);
        this.onClick = this.onClick.bind(this);
    }

    itemSellable(itemName: string) {
        return this.props.prices[itemName] > 0 || itemName === ItemsName.coins
    }

    sellDialogue() {
        let name = this.state.item.name;
        return DialogueStandards.INPUT_DIALOGUE_ELEMENT_BODY(
            "Selling",
            <div>
                <div className={"sell-inventory-title"}>How many {Strings.decamelizeLower(name)} do you wish to sell?</div>
                {this.valueDialogueCraftingDisplay({
                    input: [Object.assign({},this.state.item,{amount: 1})],
                    output: [new Items(ItemsName.coins, this.props.prices[this.state.item.name])],
                    time: 0,
                    show: ()=>true
                },this.state.inputValue)}
            </div>,
            (value: number) => {
                let item: Item = Object.assign({}, this.state.item);
                item.amount = value;
                if (value > 0 && this.checkItem(name, value)) {
                    Fetch.put("inventory/sell", item)
                        .then(response => response.json())
                        .then((data: Item) => {
                            let outputName = data.name;
                            let amount = data.amount;
                            this.removeItem(name, value);
                            let item = ItemFactory.create(outputName, amount);
                            this.props.increaseInventory(item);
                            this.closeDialogue();
                        })
                        .catch(error => {
                            console.log(`Error: ${error}`);
                            this.setState({
                                dialogue: Dialogue.Error,
                                error: `You don't have ${value} ${name}`
                            });

                        });
                    this.closeDialogue();
                } else {
                    this.setState({
                        dialogue: Dialogue.Error,
                        error: `You dont have ${value} ${name}`
                    });
                }
            },
            () => {this.closeDialogue()},
            (value)=>{this.setState({inputValue: value})},
            this.state.inputValue,
            (this.state.item.category == Category.fruit)?this.state.item.amount-1:this.state.item.amount
        )
    }

    valueDialogueCraftingDisplay(processData: ProcessData, amount: number){
        return (
            <div className="recipe">
                <div className="input items">
                    {processData.input.map((item: Item, index: number)=>
                        <div className="recipe-item" key={index}>
                            {this.img(item.name)}
                            {this.recipeAmount(item, amount)}
                        </div>
                    )}
                </div>
                <div className="items">
                    <div className="arrow">
                        <i className="arrow-right"/>
                    </div>
                </div>
                <div className="output items">
                    {processData.output.map((item: Item, index: number)=>
                        <div className="recipe-item" key={index}>
                            {this.img(item.name)}
                            {this.recipeAmount(item, amount)}
                        </div>
                    )}
                </div>
            </div>
        )
    }

    recipeAmount(item: Item, amount: number){
        let totalAmount: number = amount*item.amount;
        return <span className="recipe-item-amount">{Number.isInteger(totalAmount)?`×${totalAmount.toLocaleString()}`:`×${Math.floor(totalAmount).toLocaleString()} (${(Math.round(totalAmount * 100) / 100).toLocaleString()})`}</span>
    }

    img = DialogueTools.imageFunction("sell-inventory");

    checkItem(name: string, amount:number = 1): boolean{
        return InventoryTools.checkItem(name, amount, this.props.inventory)
    }

    removeItem(name: string, amount: number){
        InventoryTools.removeItem(name, amount, this.props.increaseInventory)
    }

    closeDialogue() {
        this.setState({
            dialogue: Dialogue.None
        });
    }

    showDialogue() {
        switch (this.state.dialogue) {
            case Dialogue.None:
                return null;
            case Dialogue.Sell:
                return this.sellDialogue();
            case Dialogue.Error:
                return this.errorDialogue();
        }
    }

    errorDialogue() {
        return DialogueStandards.CONFIRM_DIALOGUE(
            Strings.decamelize(this.state.item.name),
            this.state.error,
            () => this.closeDialogue()
        );
    }

    onClick(item: Item) {
        if(item.name == ItemsName.coins){
            return
        }
        this.setState({
            item: item,
            dialogue: Dialogue.Sell
        })
    }

    render = () => (
        <div className={"SellInventory"}>
            <span className="sell-inventory-text sell-inventory-text-sell">Sell</span>
            <div className={"sell-inventory"}>
                {Object.keys(this.props.inventory)
                    .filter(key => this.props.inventory[key].amount > 0)
                    .filter(key => this.itemSellable(key))
                    .map((key, i) => <ItemBox item={this.props.inventory[key]} key={i} sell={this.onClick}/>)}
            </div>
            {this.showDialogue()}
            <span className="sell-inventory-text sell-inventory-text-buy">Buy</span>
            <SubInventory />
        </div>
    )
}

const SellInventory = connect(mapStateToProps, mapDispatchToProps)(ConnectedSellInventory);

export default SellInventory;