import React, {Component, CSSProperties} from "react";
import Button from "./dialogue/Button";
import {connect} from "react-redux";
import {DialogueTools, Fetch, InventoryList, QuestList, Strings} from "./utils";
import {Mine} from "./mining/Mine";
import {increaseInventory, newMine, plantSeeds, plantTrees, unlockSeed} from "./actions/actions";
import {DialogueStandards} from "./dialogue/DialogueStandards";
import {Item} from "./Items/Item";
import {Items, ItemsName} from "./Items/Items";
import {Plant} from "./farming/Plant";
import {Crop, CropName} from "./Items/Crops";
import './SubInventory.css';
import {Tree, TreeName} from "./woodcutting/Tree";
import {BuyUpgradeFactory} from "./factories/BuyUpgradeFactory";
import {Fruit, FruitName} from "./Items/Fruit";

enum Dialog {
    None,
    BuyMine,
    BuyFarmPlot,
    BuyTreePatch,
}

interface ISubInventoryState {
    dialogue: Dialog,
    minePrice: number,
    farmPlotPrice: number,
    treePatchPrice: number,
}

interface ISubInventoryProps {
    inventory: InventoryList,
    newMines: (mines: Mine[]) => void,
    mines: Mine[],
    farmPlots: Plant[],
    treePatches: Tree[],
    currentQuests: QuestList,
    increaseInventory: (item: Item) => void,
    unlockedSeeds: Plant[]
    plantSeeds: (plants: Plant[]) => void,
    plantTrees: (trees: Tree[])=> void,
    unlockSeed: (seed: Plant) => void,
}

const mapStateToProps = (state: any) => {
    return {
        inventory: state.inventory,
        mines: state.mines,
        currentQuests: state.currentQuests,
        farmPlots: state.farmPlots,
        treePatches: state.treePatches,
        unlockedSeeds: state.unlockedSeeds
    }
};

function mapDispatchToProps(dispatch: any) {
    return {
        newMines: (mines: Mine[]) => dispatch(newMine(mines)),
        increaseInventory: (item: Item) => dispatch(increaseInventory(item)),
        plantSeeds: (plants: Plant[]) => dispatch(plantSeeds(plants)),
        plantTrees: (trees: Tree[]) => dispatch(plantTrees(trees)),
        unlockSeed: (seed: Plant) => dispatch(unlockSeed(seed)),
    }
}

class ConnectedSubInventory extends Component<ISubInventoryProps, ISubInventoryState> {
    private numberOfAllowedMines = 5;
    private numberOfAllowedFarmPlots = 5;
    private numberOfAllowedTreePatches = 5;

    state = {
        dialogue: Dialog.None,
        minePrice: -1,
        farmPlotPrice: -1,
        treePatchPrice: -1,
    };

    constructor(props: ISubInventoryProps) {
        super(props);
        this.buyMineClick = this.buyMineClick.bind(this);
        this.buyFarmPlotClick = this.buyFarmPlotClick.bind(this);
        this.buyTreePatchClick = this.buyTreePatchClick.bind(this);
    }

    componentDidMount() {
        let minePrice = 0;
        switch (this.props.mines.length) {
            case 0:
                minePrice = 10;
                break;
            case 1:
                minePrice = 5e2;
                break;
            case 2:
                minePrice = 5e3;
                break;
            case 3:
                minePrice = 5e4;
                break;
            case 4:
                minePrice = 5e5;
                break;
            default:
                minePrice = -1;
                break;
        }
        this.setState({
            minePrice: minePrice
        });

        let farmPlotPrice = 0;
        switch (this.props.farmPlots.length) {
            case 0:
                farmPlotPrice = 10;
                break;
            case 1:
                farmPlotPrice = 5e2;
                break;
            case 2:
                farmPlotPrice = 5e3;
                break;
            case 3:
                farmPlotPrice = 5e4;
                break;
            case 4:
                farmPlotPrice = 5e5;
                break;
            default:
                farmPlotPrice = -1;
                break;
        }
        this.setState({
            farmPlotPrice: farmPlotPrice
        });

        let treePatchPrice = 0;
        switch (this.props.treePatches.length) {
            case 0:
                treePatchPrice = 125;
                break;
            case 1:
                treePatchPrice = 5e2;
                break;
            case 2:
                treePatchPrice = 5e3;
                break;
            case 3:
                treePatchPrice = 5e4;
                break;
            case 4:
                treePatchPrice = 5e5;
                break;
            default:
                treePatchPrice = -1;
                break;
        }
        this.setState({
            treePatchPrice: treePatchPrice
        });
    }

    showDialogue() {
        switch (this.state.dialogue) {
            case Dialog.None:
                return null;
            case Dialog.BuyMine:
                return this.buyMineDialogue();
            case Dialog.BuyFarmPlot:
                return this.buyFarmPlotDialogue();
            case Dialog.BuyTreePatch:
                return this.buyTreePatchDialogue();
        }
    }

    closeDialogue() {
        this.setState({
            dialogue: Dialog.None
        });
    }

    buyMineDialogue() {
        if (this.props.inventory[ItemsName.coins].amount < this.state.minePrice) {
            return this.notEnoughMoneyDialogue("mine", this.state.minePrice);
        }
        if (this.props.mines.length < this.numberOfAllowedMines) {
            return DialogueStandards.YES_NO_DIALOGUE(
                "Buy mine",
                `Do you want to buy a mine? It costs ${this.state.minePrice} coins.`,
                () => {
                    this.buyMineClick();
                    this.closeDialogue()
                },
                () => {
                    this.closeDialogue()
                }
            );
        }
    }

    buyMineClick() {
        Fetch.post("/api/mining/buymine", []).then(response => {
            switch (response.status) {
                case 200:
                    response.json().then(result => {
                        let id = result[0];
                        let timeToMine = result[1];
                        let finishTime = result[2];
                        this.props.newMines(
                            [
                                new Mine(id, timeToMine, 3, finishTime)
                            ]
                        );
                        this.props.increaseInventory(new Items(ItemsName.coins, -this.state.minePrice));
                        let minePrice = 0;
                        switch (this.props.mines.length) {
                            case 0:
                                minePrice = 10;
                                break;
                            case 1:
                                minePrice = 5e2;
                                break;
                            case 2:
                                minePrice = 5e3;
                                break;
                            case 3:
                                minePrice = 5e4;
                                break;
                            case 4:
                                minePrice = 5e5;
                                break;
                            default:
                                minePrice = -1;
                                break;
                        }
                        this.setState({
                            minePrice: minePrice
                        });
                    }).catch(error => {
                        console.error("Error: " + error);
                    });
                    break;

                default:
                    console.warn(response.status + " " + response.statusText);
            }
        }).catch(error => {
            console.error("Error: " + error);
        });
    }

    buyFarmPlotDialogue() {
        if (this.props.inventory[ItemsName.coins].amount < this.state.farmPlotPrice) {
            return this.notEnoughMoneyDialogue("farm plot", this.state.farmPlotPrice);
        }
        if (this.props.farmPlots.length < this.numberOfAllowedFarmPlots) {

            return DialogueStandards.YES_NO_DIALOGUE(
                "Buy farm plot",
                `Do you want to buy a farm plot? It costs ${this.state.farmPlotPrice} coins.`,
                () => {
                    this.buyFarmPlotClick();
                    this.closeDialogue()
                },
                () => this.closeDialogue()
            )
        }
    }

    buyFarmPlotClick() {
        // this.props.increaseInventory(new Items(ItemsName.coins, -this.state.farmPlotPrice));
        // this.props.plantSeeds([new Plant(CropName.empty, 0)]);
        Fetch.post("/farmPlots/buyNewFarmPlot", []).then(response => {
            switch (response.status) {
                case 200:
                    this.props.plantSeeds(
                        [
                            new Plant(CropName.empty, this.props.farmPlots.length)
                        ]
                    );
                    this.props.increaseInventory(new Items(ItemsName.coins, -this.state.farmPlotPrice));
                    let farmPlotPrice = 0;
                    switch (this.props.farmPlots.length) {
                        case 0:
                            farmPlotPrice = 10;
                            break;
                        case 1:
                            farmPlotPrice = 5e2;
                            break;
                        case 2:
                            farmPlotPrice = 5e3;
                            break;
                        case 3:
                            farmPlotPrice = 5e4;
                            break;
                        case 4:
                            farmPlotPrice = 5e5;
                            break;
                        default:
                            farmPlotPrice = -1;
                            break;
                    }
                    this.setState({
                        farmPlotPrice: farmPlotPrice
                    });
                    break;
                default:
                    console.log(response.status + " " + response.statusText);
            }
        }).catch(error => {
            console.error(error);
        })
    }

    buyTreePatchDialogue() {
        if (this.props.inventory[ItemsName.coins].amount < this.state.treePatchPrice) {
            return this.notEnoughMoneyDialogue("farm plot", this.state.treePatchPrice);
        }
        if (this.props.treePatches.length < this.numberOfAllowedTreePatches) {

            return DialogueStandards.YES_NO_DIALOGUE(
                "Buy tree patch",
                `Do you want to buy a new tree patch? It costs ${this.state.treePatchPrice} coins.`,
                () => {
                    this.buyTreePatchClick();
                    this.closeDialogue()
                },
                () => this.closeDialogue()
            )
        }
    }

    buyTreePatchClick() {
        Fetch.post("/treePatches/buyNewTreePatch", []).then(response => {
            switch (response.status) {
                case 200:
                    this.props.plantTrees(
                        [
                            new Tree(TreeName.empty, this.props.treePatches.length)
                        ]
                    );
                    this.props.increaseInventory(new Items(ItemsName.coins, -this.state.treePatchPrice));
                    let treePatchPrice = 0;
                    switch (this.props.treePatches.length) {
                        case 0:
                            treePatchPrice = 125;
                            break;
                        case 1:
                            treePatchPrice = 5e2;
                            break;
                        case 2:
                            treePatchPrice = 5e3;
                            break;
                        case 3:
                            treePatchPrice = 5e4;
                            break;
                        case 4:
                            treePatchPrice = 5e5;
                            break;
                        default:
                            treePatchPrice = -1;
                            break;
                    }
                    this.setState({
                        treePatchPrice: treePatchPrice
                    });
                    break;
                default:
                    console.log(response.status + " " + response.statusText);
            }
        }).catch(error => {
            console.error(error);
        })
    }

    showBuyUpgradeWoodcutting() {
        let name = "fruits";
        let upgradeData = {
            apple: {
                requirements: [
                ],
                price: [
                    new Items(ItemsName.coins, 150)
                ]
            },
            peach: {
                requirements: [
                    new Fruit(FruitName.apple, 250)
                ],
                price: [
                    new Items(ItemsName.coins, 1000)
                ]
            },
            orange: {
                requirements: [
                    new Fruit(FruitName.apple, 500),
                    new Fruit(FruitName.peach, 250)
                ],
                price: [
                    new Fruit(FruitName.apple, 200),
                    new Fruit(FruitName.peach, 100),
                    new Items(ItemsName.coins, 2000)
                ]
            },
            chestnut: {
                requirements: [
                    new Fruit(FruitName.peach, 500),
                    new Fruit(FruitName.orange, 2000),
                ],
                price: [
                    new Fruit(FruitName.peach, 200),
                    new Fruit(FruitName.orange, 500),
                    new Items(ItemsName.coins, 3000)
                ]
            }
        };
        let show = (name: string)=>{
            return !(this.props.inventory.hasOwnProperty(name) && this.props.inventory[name].amount >= 1)
        };
        let unlock = (name: string)=>{
            this.props.increaseInventory(new Fruit(name as FruitName, 1))
        };
        show = show.bind(this);
        unlock = unlock.bind(this);
        return BuyUpgradeFactory.create(name, upgradeData, show, unlock);
    }

    showBuyUpgradeFarming() {
        let name = "seeds";
        let upgradeData = {
            lettuce: {
                requirements: [
                    new Crop(CropName.wheat, 15000),
                ],
                price: [
                    new Items(ItemsName.coins, 300)
                ]
            },
            cucumber: {
                requirements: [
                    new Crop(CropName.lettuce, 2000),

                ],
                price: [
                    new Crop(CropName.lettuce, 500),
                    new Items(ItemsName.coins, 500)
                ]
            },
            barley: {
                requirements: [
                    new Crop(CropName.wheat, 30000),

                ],
                price: [
                    new Crop(CropName.wheat, 10000),
                    new Items(ItemsName.coins, 2000             
                    )
                ]
            },
            carrot: {
                requirements: [
                    new Crop(CropName.lettuce, 3000),
                    new Crop(CropName.cucumber, 1000),

                ],
                price: [
                    new Crop(CropName.lettuce, 1000),
                    new Crop(CropName.cucumber, 500),
                    new Items(ItemsName.coins, 3000)
                ]
            },/*
            olives: {
                requirements: [
                    new Crop(CropName.lettuce, 20000),
                    new Crop(CropName.barley, 15000),
                    new Items(ItemsName.coins, 400)
                ]
            },
            grape: {
                requirements: [
                    new Crop(CropName.barley, 20000),
                    new Crop(CropName.olives, 15000),
                    new Items(ItemsName.coins, 550)
                ]
            },
            */
        };
        let show = (name: string)=>{
            for(let key in this.props.unlockedSeeds){
                if(this.props.unlockedSeeds[key].name == name){
                    return false
                }
            }
            return true;
        };
        let unlock = (name: string)=>{
            this.props.unlockSeed(new Plant(name as CropName))
        };
        show = show.bind(this);
        unlock = unlock.bind(this);
        return BuyUpgradeFactory.create(name, upgradeData, show, unlock);
    }

    notEnoughMoneyDialogue(titleItem: string, itemPrice: number) {
        return DialogueStandards.CONFIRM_DIALOGUE(
            `Buy ${titleItem}`,
            `You don't have enough coins. It costs ${itemPrice} coins`,
            () => this.closeDialogue()
        )
    }


    plotImg = (name: string, dialog: Dialog) => <img
        src={`images/${name}.png`}
        alt={Strings.decamelize(name)}
        className={`sub-inventory-image`}
        onClick={() => this.setState({dialogue: dialog})}
    />

    buyMine() {
        return (
            <div className={"sub-inventory-buy"} >
                {this.plotImg("mine", Dialog.BuyMine)} <br/>
                <span className={"sub-inventory-buy-mine-price-tag"}>Price: {this.state.minePrice.toLocaleString()} coins</span> <br/>
                <Button text={"Buy mine"} onClick={() => this.setState({dialogue: Dialog.BuyMine})}/>
            </div>
        );
    }
    buyFarmPlot() {
        return (
            <div className={"sub-inventory-buy"}>
                {this.plotImg("farmplot", Dialog.BuyFarmPlot)}
                <span className={"sub-inventory-buy-farm-plot-price-tag"}>Price: {this.state.farmPlotPrice.toLocaleString()} coins</span>
                <br/>
                <Button text={"Buy farm plot"} onClick={() => this.setState({dialogue: Dialog.BuyFarmPlot})}/>
            </div>
        );
    }
    buyTreePatch() {
        return (
            <div className={"sub-inventory-buy"}>
                {this.plotImg("treepatch", Dialog.BuyTreePatch)} <br/>
                <span className={"sub-inventory-buy-tree-patch-price-tag"}>Price: {this.state.treePatchPrice.toLocaleString()} coins</span>
                <br/>
                <Button text={"Buy tree patch"} onClick={() => this.setState({dialogue: Dialog.BuyTreePatch})}/>
            </div>
        );
    }

    render() {
        return (
            <div className={"SubInventory"}>
                {this.showDialogue()}
                <div className={"buy-plots"}>
                    {this.buyMine()}
                    {this.buyFarmPlot()}
                    {this.buyTreePatch()}
                </div>
                <div className={"buy-upgrades"}>
                    {this.showBuyUpgradeFarming()}
                    {this.showBuyUpgradeWoodcutting()}
                </div>
            </div>
        )
    }
}

const SubInventory = connect(mapStateToProps, mapDispatchToProps)(ConnectedSubInventory);

export default SubInventory;