import React, {Component} from 'react';
import './TabBar.css';
import {Tab} from './Enum'
import {connect} from "react-redux";
import {ReduxState} from "./reducers/reducer";
import {Plant} from "./farming/Plant";
import {CropName} from "./Items/Crops";
import {ProcessList, Time} from "./utils";
import {Mine} from "./mining/Mine";
import {Tree} from "./woodcutting/Tree";

interface ITabBarProps{
    setTab: (tab: Tab)=>void,
    showTab: IShowTab,
    readyTab: IReadyTab,
    rulesAccepted: boolean
}

interface ITabBarState{
    key: number
}

interface IShowTab{
    [key:string]: boolean
}

interface IReadyTab{
    [key:string]: ()=>boolean
}
function mapStateToProps(state: ReduxState){
    return  {
        showTab: {
            quest: Object.keys(state.currentQuests).length>0,
            farming: state.farmPlots.length>0,
            mining: state.mines.length>0,
            woodcutting: state.treePatches.length>0,
            sellInventory: Object.keys(state.inventory).filter(a=>state.prices[a]>0).length>0 || state.mines.length>0,
            scoreboard : true,
            settings: true,
            feedback: !state.feedback.finished,
        },
        readyTab: {
            quest: ()=>false,
            farming: ()=>checkFarmplots(state.farmPlots, state.unlockedSeeds, state.processing),
            mining: ()=>checkMines(state.mines, state.processing),
            woodcutting: ()=>checkTreePatches(state.treePatches, state.processing),
            sellInventory: ()=>false,
            scoreboard : ()=>false,
            settings: ()=>false,
            feedback: ()=>true,
        }

    }
}

function checkFarmplots(farmplots: Plant[], unlockedSeeds: Plant[], processing: ProcessList): boolean{
    if(processing.mill && Time.hasHappened(processing.mill.finishTime)){
        return true;
    }
    if(processing.baker && Time.hasHappened(processing.baker.finishTime)){
        return true;
    }
    for(let i in farmplots){
        let plant = farmplots[i];
        if(plant.name === CropName.empty && unlockedSeeds.length>0){
            return true;
        }
        if(Time.hasHappened(plant.finishTime) && plant.name !== CropName.empty){
           return true;
        }
    }
    return false;
}

function checkMines(mines: Mine[], processing: ProcessList): boolean{
    if(processing.foundry && Time.hasHappened(processing.foundry.finishTime)){
        return true;
    }
    if(processing.forge && Time.hasHappened(processing.forge.finishTime)){
        return true;
    }
    for(let i in mines){
        let mine = mines[i];
        if(mine.stage(mine.timeLeft())>0){
            return true;
        }
    }
    return false;
}

function checkTreePatches(trees: Tree[], processing: ProcessList): boolean{
    if(processing.woodWorker && Time.hasHappened(processing.woodWorker.finishTime)){
        return true;
    }
    for(let i in trees){
        let tree = trees[i];
        if(tree.timeLeftFruit()<=0){
            return true;
        }
    }
    return false;
}

function mapDispatchToProps(dispatch: any) {
    return {
    };
}

class ConnectedTabBar extends Component<ITabBarProps,ITabBarState>{
    state = {
        key: Math.random()
    };
    interval: any = null;

    constructor(props: ITabBarProps) {
        super(props);
        this.interval = setInterval(()=>this.setState({key:Math.random()}),1000)
    }

    componentWillUnmount(): void {
        clearInterval(this.interval);
    }

    componentDidUpdate(prevProps: Readonly<ITabBarProps>, prevState: Readonly<ITabBarState>, snapshot?: any): void {
        let amount = 0;
        for(let key in this.props.readyTab){
            if(this.props.showTab[key] && this.props.readyTab[key]()){
                amount++
            }
        }
        document.title = (amount>0?`(${amount}) `:"")+ "Merchants of Rome";
        return;
    }

    componentDidMount() {
    }

    mapToImage(tab: Tab){
        switch(tab){
            case Tab.mining:
                return "pickaxe";
            case Tab.woodcutting:
                return "axe";
            case Tab.farming:
                return "wateringCan";
            default:
                return tab.toString()
        }
    }

    showNotification(tab: string){
        if(tab == Tab.settings && !this.props.rulesAccepted){
            return <i className={"tab-bar-notification"}> </i>
        }
        if(this.props.readyTab.hasOwnProperty(tab) && this.props.readyTab[tab]()){
            return <i className={"tab-bar-notification"}> </i>
        }
        return null
    }

    render() {
        return (
            <div className="tab-bar">
                {Object.keys(Tab)
                    .filter((a: string)=>{
                        if(this.props.showTab.hasOwnProperty(a)){
                            return this.props.showTab[a];
                        }
                        return true;
                    })
                    .map((a: string,i)=>
                    <div className={"tab-bar-image-div"} key={i}>
                        <img
                        className="tab-bar-image"
                        id={"tab-bar-"+a.toString()}
                        src={`images/${this.mapToImage(a as Tab)}.png`}
                        alt={a.toString()}
                        onClick={()=>this.props.setTab(a as Tab)}
                        />
                        {this.showNotification(a)}
                    </div>)
                }
            </div>
        )
    }
}

const TabBar = connect(mapStateToProps, mapDispatchToProps)(ConnectedTabBar);

export default TabBar;
