import React, {Component} from "react";
import {connect} from "react-redux";
import './BuyUpgradeFactory.css'
import {ReduxState} from "../reducers/reducer";
import {
    BuyUpgradeData,
    BuyUpgradeDataList,
    DialogueTools, Fetch,
    HistoryMap,
    InventoryList,
    InventoryTools,
    ProcessData,
    Strings
} from "../utils";
import {increaseInventory, setUnlockedSeeds, unlockOre, unlockSeed} from "../actions/actions";
import {Plant} from "../farming/Plant";
import {Ore} from "../Items/Ore";
import {Item} from "../Items/Item";
import {DialogueStandards} from "../dialogue/DialogueStandards";

enum Dialogue {
    None,
    Buy,
    Confirm,
    Error
}

interface IBuyUpgradeProps {
    name: string,
    buyUpgradeDataList: BuyUpgradeDataList,
    increaseInventory: (item: Item) => void,
    inventory: InventoryList
    history: HistoryMap
    show: (name: string)=> boolean
    unlock: (name: string)=> void
}

interface IBuyUpgradeState {
    timeLeft: number,
    dialogue: Dialogue,
    error: string
}

export class BuyUpgradeFactory {

    static create(name: string, buyUpgradeDataList: BuyUpgradeDataList, show: (name: string) => boolean, unlock: (name: string) => void): JSX.Element {
        let UpgradeItem = connect(this.mstpFactory(), this.mdtpFactory())(ConnectedBuyUpgrade);
        return <UpgradeItem name={name} buyUpgradeDataList={buyUpgradeDataList} show={show} unlock={unlock}/>
    }

    static mstpFactory = () => {
        return (state: ReduxState) => {
            return {
                history: state.history,
                inventory: state.inventory,
            }
        }
    };

    static mdtpFactory = () => {
        return (dispatch: any) => {
            return {
                unlockSeed: (seed: Plant) => dispatch(unlockSeed(seed)),
                unclockOre: (ore: Ore) => dispatch(unlockOre(ore)),
                increaseInventory: (item: Item) => dispatch(increaseInventory(item))
            }
        }
    };
}

class ConnectedBuyUpgrade extends Component<IBuyUpgradeProps, IBuyUpgradeState> {

    private interval: any;

    state = {
        timeLeft: 0,
        dialogue: Dialogue.None,
        requirements: [],
        error: "Unspecified error"
    };

    constructor(props: IBuyUpgradeProps) {
        super(props);
        this.onClick = this.onClick.bind(this);
    }

    componentDidMount(): void {
        this.interval = setInterval(()=> this.tick(), 1000);
    }

    componentWillUnmount() {
        clearInterval(this.interval);
    }

    tick(){
        if(this.state.timeLeft <= 0){
            return;
        }

        this.setState(state=>({
            timeLeft: state.timeLeft - 1
        }))
    }

    getItem(name: string){
        return this.props.inventory[name];
    }

    checkItem(name: string, amount:number = 1): boolean{
        return InventoryTools.checkItem(name, amount, this.props.inventory)
    }

    getCategory(){
        let category = "";
        switch(this.props.name){
            case "seeds":
                category = "plants";break;
            case "fruits":
                category = "trees";break;
        }
        return category
    }

    getVerb(){
        let verb = "";
        switch(this.props.name){
            case "seeds":
                verb = "harvest";break;
            case "fruits":
                verb = "harvest";break;
        }
        return verb
    }



    getItemTotal(name: string): number{
        let category = this.getCategory();
        return this.props.history[category][name];
    }

    checkItemTotal(name: string, amount:number = 1): boolean{
        let category = this.getCategory();
        return InventoryTools.checkItemTotal(category, name, amount, this.props.history)
    }

    removeItem(name: string, amount: number){
        InventoryTools.removeItem(name, amount, this.props.increaseInventory)
    }

    img(name: string){
        return DialogueTools.imageFunction("buy-upgrade")(name);
    }

    getNextUpgrade(){
        for(let key in this.props.buyUpgradeDataList){
            if(this.props.show(key)){
                return key
            }
        }
        return "nothing"
    }

    getNextUpgradeDialogue(nextUpgrade: string){
        if(nextUpgrade === "nothing"){
            return <div>I unfortunately have nothing new to offer you at this time</div>
        }else{
            let item = this.props.name=="seeds"?`packet of ${nextUpgrade} seeds`:Strings.decamelizeLower(nextUpgrade);
            return <div>
                I can offer you
                {["a","e","i","o"].includes(item.substring(0,1).toLowerCase())?" an ":" a "}
                {item}
            <br/>
                {this.props.buyUpgradeDataList[nextUpgrade].requirements.length>0?"But there are some requirements":""}
            </div>
        }
    }

    requirementAndPriceDisplay(nextUpgrade: string, buyUpgradeData: BuyUpgradeData){
        if(!buyUpgradeData){
            return null;
        }
        return (
            <div className="recipe">
                <div className="input items">
                    {buyUpgradeData.requirements.length>0?Strings.decamelize(this.getVerb()+"Requirements"):""}
                    {buyUpgradeData.requirements.map((item: Item, index: number)=>
                        <div className="recipe-item" key={index}>
                            {this.img(item.name)}
                            <span className={`recipe-item-amount ${this.checkItemTotal(item.name,item.amount)?"requirement-passed":"requirement-failed"}`}>{this.getItemTotal(item.name)?this.getItemTotal(item.name):0}/{item.amount}</span>
                        </div>
                    )}
                </div>
                <div className="items">
                    {this.img(`${nextUpgrade}${this.props.name=="seeds"?"Seeds":""}`)}
                </div>
                <div className="output items">
                    Price:
                    {buyUpgradeData.price.map((item: Item, index: number)=>
                        <div className="recipe-item" key={index}>
                            {this.img(item.name)}
                            <span className={`recipe-item-amount ${this.checkItem(item.name,item.amount)?"requirement-passed":"requirement-failed"}`}>{this.getItem(item.name)?this.getItem(item.name).amount:0}/{item.amount}</span>
                        </div>
                    )}
                </div>
            </div>
        )
    }

    buyUpgradeDialogue() {
        let nextUpgrade = this.getNextUpgrade();
        return DialogueStandards.YES_NO_DIALOGUE_ELEMENT_BODY(
            Strings.decamelize(this.props.name+"Salesman"),
            <div className={"buy-upgrade-dialogue-div"}>
                <div className="buy-upgrade-dialogue-top-text">
                    Do you want some better {this.props.name}?<br/>
                    {this.getNextUpgradeDialogue(nextUpgrade)}
                </div>
                {this.requirementAndPriceDisplay(nextUpgrade,this.props.buyUpgradeDataList[nextUpgrade])}
            </div>,
            () => {this.buyUpgrade(nextUpgrade)},
            () => {this.closeDialogue()}
        )
    }

    checkRequirements(upgrade: string){
        let data = this.props.buyUpgradeDataList[upgrade];
        for(let key in data.requirements){
            let item = data.requirements[key];
            if(!this.checkItemTotal(item.name, item.amount))
                return item.name;
        }
        return "passed"
    }

    checkPrices(upgrade: string){
        let data = this.props.buyUpgradeDataList[upgrade];
        for(let key in data.price){
            let item = data.price[key];
            if(!this.checkItem(item.name, item.amount))
                return item.name;
        }
        return "passed"
    }

    buyUpgrade(nextUpgrade: string) {
        if(!this.props.buyUpgradeDataList[nextUpgrade]){
            this.closeDialogue();
            return null;
        }
        let req = this.checkRequirements(nextUpgrade);
        if(req!=="passed"){
            this.setState({
                error: `You don't pass the ${this.getVerb()} requirements for ${req}`,
                dialogue: Dialogue.Error
            });
            return;
        }
        let price = this.checkPrices(nextUpgrade);
        if(price!=="passed"){
            this.setState({
                error: `You don't have enough ${price} to buy this`,
                dialogue: Dialogue.Error
            });
            return;
        }
        this.setState({dialogue: Dialogue.Confirm})
    }

    onClick() {
        if (this.state.dialogue != Dialogue.None) {
            return;
        }
        this.setState({dialogue: Dialogue.Buy})
    }

    buy(upgrade: string){
        let data = this.props.buyUpgradeDataList[upgrade];
        for(let key in data.price){
            let item = data.price[key];
            this.removeItem(item.name, item.amount);
        }
        this.props.unlock(upgrade);
        Fetch.put(`upgrade/${this.props.name}/${upgrade}`).then(()=>{});
    }

    confirmDialogue(){
        let nextUpgrade = this.getNextUpgrade();
        return DialogueStandards.YES_NO_DIALOGUE_ELEMENT_BODY(
            Strings.decamelize(this.props.name+"Salesman"),
            <div>
                Are you sure you want to buy this upgrade?
            </div>,
            ()=>{this.buy(nextUpgrade);this.closeDialogue()},
            ()=>{this.closeDialogue()}
        )
    }

    showDialogue() {
        switch (this.state.dialogue) {
            case Dialogue.None:
                return null;
            case Dialogue.Buy:
                return this.buyUpgradeDialogue();
            case Dialogue.Confirm:
                return this.confirmDialogue();
            case Dialogue.Error:
                return this.errorDialogue();
        }
    }

    closeDialogue() {
        this.setState({
            dialogue: Dialogue.None
        });
    }

    errorDialogue(){
        return DialogueStandards.CONFIRM_DIALOGUE(
            Strings.decamelize(this.props.name+"Salesman"),
            this.state.error,
            ()=>this.closeDialogue()
        )
    }

    render() {
        return (
            <div className={`buy-upgrade ${this.props.name}`}>
                {this.showDialogue()}
                <img className={"buy-upgrade-image-shop"} src={`images/${this.props.name}Salesman.png`} alt={Strings.decamelize(this.props.name)} onClick={this.onClick} />

            </div>
        )
    }
    //{<span className={"buy-upgrade-requirement"}>{Object.keys(this.props.buyUpgradeDataList).map((key, index: number) => this.props.buyUpgradeDataList[key].requirements[index])}</span>}
}