import {Component} from "react";
import {DialogueStandards} from "../dialogue/DialogueStandards";
import React from "react";
import {connect} from "react-redux";
import {
    increaseInventory,
    plantSeeds, removeProcess, setProcess,
    setTab,
    unlockSeed
} from "../actions/actions";
import './ProcessingFactory.css'
import {Item} from "../Items/Item";
import {Tab} from "../Enum";
import {
    DialogueTools, Fetch,
    InventoryList,
    InventoryTools,
    Process,
    ProcessData,
    ProcessDataList,
    Strings,
    Time
} from "../utils";
import {ReduxState} from "../reducers/reducer";

enum Dialogue{
    None,
    Collect,
    Error,
    Selection,
    Value
}

interface IProcessingPlantProps{
    process: Process
    inventory: InventoryList
    name: string
    processData: ProcessDataList
    setProcess: (category: string, name: string, timeLeft: number, amount: number, finishTime?: Date) => void
    removeProcess: (name: string) => void
    increaseInventory: (item: Item) => void,
}

interface IProcessingPlantState{
    timeLeft: number
    dialogue: Dialogue
    chosen: number
    error: string
    inputValue: number
}

export interface IProcessingPlantTools{
    setTab: (tab: Tab) => void,
    increaseInventory: (item: Item) => void,
}

export class ProcessingPlantFactory{
    static create(name: string, processData: ProcessDataList): JSX.Element{
        let ProcessingPlant = connect(this.mstpFactory(name), this.mdtpFactory())(ConnectedProcessingPlant);
        return <ProcessingPlant name = {name} processData = {processData} />
    }

    static mstpFactory = (name: string) => {
        return (state: ReduxState) => {
            return {
                process: state.processing[name],
                inventory: state.inventory
            }
        };
    };

    static mdtpFactory = () => {
        return (dispatch: any) => {
            return {
                setProcess: (category: string, name: string, timeLeft: number, amount: number, finishTime?: Date) =>
                    dispatch(setProcess(category, name, timeLeft, amount, finishTime)),
                removeProcess: (name: string) => dispatch(removeProcess(name)),
                increaseInventory: (item: Item) => dispatch(increaseInventory(item)),
            }
        };
    };


}

class ConnectedProcessingPlant extends Component<IProcessingPlantProps,IProcessingPlantState>{
    private interval: any;

    state = {
        timeLeft: 0,
        dialogue: Dialogue.None,
        chosen: 0,
        error: "Unspecified error",
        inputValue: 1
    };

    constructor(props: IProcessingPlantProps) {
        super(props);
        if(this.props.process) {
            this.state.timeLeft = Time.timeLeft(this.props.process.finishTime);
        }
        this.onClick = this.onClick.bind(this)
    }

    componentDidMount(): void {
        this.interval = setInterval(()=> this.tick(), 1000);
        if(this.props.process) {
            this.setState({
                timeLeft: Time.timeLeft(this.props.process.finishTime)
            })
        }
    }

    tick(){
        if(this.state.timeLeft<=0){
            return;
        }

        this.setState(state=>({
            timeLeft: state.timeLeft-1
        }))
    }
    componentWillUnmount() {
        clearInterval(this.interval);
    }

    showDialogue(){
        switch(this.state.dialogue){
            case Dialogue.None:
                return null;
            case Dialogue.Value:
                return this.valueDialogue();
            case Dialogue.Collect:
                return this.collectDialogue();
            case Dialogue.Error:
                return this.errorDialogue();
            case Dialogue.Selection:
                return this.selectDialogue();
        }
    }

    closeDialogue(){
        this.setState({dialogue: Dialogue.None})
    }

    valueDialogue(){
        let name = Object.keys(this.props.processData)[this.state.chosen];
        let processData: ProcessData = this.props.processData[name];
        let question: string = (processData.input[0].amount === 1)?
            `How many ${Strings.decamelizeLower(processData.input[0].name)} do you wish to send to the ${Strings.decamelizeLower(this.props.name)}?`
            :
            `How many ${Strings.decamelizeLower(processData.output[0].name)} do you wish to make?`;
        let max = this.findMax(processData);
        let initialValue = Math.min(this.state.inputValue, max);
        let body: JSX.Element =
            <div>
            <div className="question">{question}</div>
                {this.valueDialogueCraftingDisplay(processData, initialValue)}
            </div>;
        return DialogueStandards.INPUT_DIALOGUE_ELEMENT_BODY(
            Strings.decamelize(this.props.name),
            body,
            (value: number)=>{
                if(value < 1){
                    return;
                }
                let checkInput: number = this.checkInput(processData.input, value);
                if(checkInput === -1){
                    this.removeInput(processData.input, value);
                    this.props.setProcess(this.props.name, name, processData.time*value, value);
                    Fetch.put(`process/start`,{
                        name: name,
                        amount: value
                    })
                        .catch(error=>console.warn(error));
                    this.setState({
                        timeLeft: processData.time*value
                    });
                    this.closeDialogue();
                }else{
                    this.setState({
                        dialogue: Dialogue.Error,
                        error: `You don't have ${processData.input[checkInput].amount*value} ${Strings.decamelizeLower(processData.input[checkInput].name)}`
                    })
                }
            },
            ()=>{this.closeDialogue()},
            (value)=>{this.setState({inputValue: value})},
            initialValue,
            max
        )
    }

    findMax(processData: ProcessData): number{
        let max = 1e9;
        for(let key in processData.input){
            let item =  this.props.inventory[processData.input[key].name];
            max = Math.min(max, item?Math.floor(item.amount/processData.input[key].amount):1);
        }
        return Math.max(max,1);
    }

    valueDialogueCraftingDisplay(processData: ProcessData, amount: number){
        return (
        <div className="recipe">
            <div className="input items">
                {processData.input.map((item: Item, index: number)=>
                    <div className="recipe-item" key={index}>
                        {this.img(item.name)}
                        {this.recipeAmount(item, amount)}
                    </div>
                )}
            </div>
            <div className="items">
                <div className="arrow">
                    <i className="arrow-right"/>
                    <span style={{marginTop: "5px"}}>{Time.timeFormat(processData.time*amount)}</span>
                </div>
            </div>
            <div className="output items">
                {processData.output.map((item: Item, index: number)=>
                    <div className="recipe-item" key={index}>
                        {this.img(item.name)}
                        {this.recipeAmount(item, amount)}
                    </div>
                )}
            </div>
        </div>
        )
    }

    recipeAmount(item: Item, amount: number){
        let totalAmount: number = amount*item.amount;
        return <span className="recipe-item-amount">{Number.isInteger(totalAmount)?`×${totalAmount.toLocaleString()}`:`×${Math.floor(totalAmount).toLocaleString()} (${(Math.round(totalAmount * 100) / 100).toLocaleString()})`}</span>
    }

    errorDialogue(){
        return DialogueStandards.CONFIRM_DIALOGUE(
            Strings.decamelize(this.props.name),
            this.state.error,
            ()=>this.closeDialogue()
            )
    }

    img(name: string){
        return DialogueTools.imageFunction("processing")(name);
    }

    checkInput(input: Item[], value: number){
        for(let key in input){
            if(!this.checkItem(input[key].name, input[key].amount*value)){
                return Number(key);
            }
        }
        return -1;
    }

    removeInput(input: Item[], value: number){
        for(let key in input){
            this.removeItem(input[key].name, input[key].amount*value)
        }
    }

    checkItem(name: string, amount:number = 1): boolean{
        return InventoryTools.checkItem(name, amount, this.props.inventory)
    }

    addItem(item: Item){
        this.props.increaseInventory(item);
    }

    removeItem(name: string, amount: number){
        InventoryTools.removeItem(name, amount, this.props.increaseInventory)
    }

    collectDialogue(){
        if(this.state.timeLeft > 0){
            return this.notReadyDialogue();
        }
        let process = this.props.processData[this.props.process.name];
        let item = Object.assign({},process.output[0]);
        item.amount *= this.props.process.amount;
        return DialogueStandards.CONFIRM_DIALOGUE_ELEMENT_BODY(
            Strings.decamelize(this.props.name),
            <div>
                You receive {this.img(item.name)}×{item.amount}
            </div>,
            ()=>{//TODO collect more than 1 item
                Fetch.put(`process/collect`,{
                    name: this.props.process.name
                })
                    .catch(error=>console.warn(error));

                this.addItem(item);
                this.props.removeProcess(this.props.name);
                this.closeDialogue()
            }
        )
    }

    selectDialogue() {
        return DialogueStandards.CONFIRM_DIALOGUE_ELEMENT_BODY(
            Strings.decamelize(this.props.name),
            <div className={"select-dialogue-div"}>
                What do you want to make? <br/>
                <div className="select-dialogue-items">
                    {Object.keys(this.getAvailableRecipes()).map((key: string, index: number) =>
                        <div className={"select-dialogue-item"} key={index} onClick={() => {this.setState({chosen: index, dialogue: Dialogue.Value})}}>
                            {this.img(this.props.processData[key].output[0].name)} <br/>
                            {Strings.decamelize(this.props.processData[key].output[0].name)}
                        </div>
                    )}
                </div>
            </div>,
            () => {this.closeDialogue()}
        )
    }

    notReadyDialogue(){
        let processData = this.props.processData[this.props.process.name];
        let name = processData.input[0].name;
        return DialogueStandards.CONFIRM_DIALOGUE_ELEMENT_BODY(
            Strings.decamelize(this.props.name),
            <div>
                The {this.props.name} has not finished processing your {Strings.decamelizeLower(name)}!
                <br />
                It will be finished in {Time.timeFormat(this.state.timeLeft)}
                {this.valueDialogueCraftingDisplay(processData, this.props.process.amount)}
            </div>,
            ()=>{this.closeDialogue()}
        )
    }

    getAvailableRecipes(){
        let newProcessData: ProcessDataList = {};
        for(let key in this.props.processData){
            if(this.props.processData[key].show()){
                newProcessData[key] = this.props.processData[key];
            }
        }
        return newProcessData;
    }

    onClick(){
        if(this.state.dialogue != Dialogue.None)
            return;
        if(this.props.process){
            this.setState({
                dialogue: Dialogue.Collect
            });
            return;
        }
        if(Object.keys(this.getAvailableRecipes()).length==0){
            this.setState({
                dialogue: Dialogue.Error
            });
            return;
        }
        if(Object.keys(this.getAvailableRecipes()).length==1){
            this.setState({
                dialogue: Dialogue.Value
            });
            return;
        }
        if(Object.keys(this.getAvailableRecipes()).length>1){
            this.setState({
                dialogue: Dialogue.Selection
            });
            return;
        }
    }


    render() {
        return (
            <div className={`processing-plant ${this.props.name}`}>
                {this.showDialogue()}
                <img className={"processing-plant-image"} src={`images/${DialogueTools.checkDLC(this.props.name)?"bakerDLC":this.props.name}.png`} alt={Strings.decamelize(this.props.name)} onClick={this.onClick}/>
                {<span className={"processing-plant-timer"}>{this.state.timeLeft>0?Time.timeFormat(this.state.timeLeft):(this.props.process)?"Finished":"Ready"}</span>}
            </div>

        )
    }


}