import React, {Component} from 'react';
import './Farm.css';
import FarmingPlot from "./FarmingPlot";
import {Plant} from "./Plant";
import {Item} from "../Items/Item";
import {Seed} from "./Seed";
import {Crop, CropName} from "../Items/Crops";
import {addHistoryData, harvestSeeds, increaseInventory, plantSeeds, uprootSeed} from "../actions/actions";
import {connect} from "react-redux";
import {DialogueStandards} from "../dialogue/DialogueStandards";
import {ProcessingPlantFactory} from "../factories/ProcessingFactory";
import {Food, FoodName} from "../Items/Food";
import {Fetch, Strings} from "../utils";
import {ItemsName} from "../Items/Items";
import {Ore, OreName} from "../Items/Ore";
import {Tree} from "../woodcutting/Tree";

enum Dialogue {
    None,
    Plant,
    Uproot
}

interface IFarmState {
    dialogue: Dialogue,
    chosenPlots: number[],
    chosenSeed: Plant,
}

interface IFarmProps {
    farmPlots: Plant[],
    unlockedSeeds: Plant[],
    plantSeeds: (plants: Plant[]) => void,
    uprootSeed: (plotId: number) => void,
    harvestSeeds: (plants: Plant[]) => void,
    increaseInventory: (item: Item) => void,
    addHistoryData: (category: string, name: string, amount: number) => void,
}

const mapStateToProps = (state: any) => {
    return {
        farmPlots: state.farmPlots,
        unlockedSeeds: state.unlockedSeeds,
    }
};

function mapDispatchToProps(dispatch: any) {
    return {
        plantSeeds: (plants: Plant[]) => dispatch(plantSeeds(plants)),
        harvestSeeds: (plants: Plant[]) => dispatch(harvestSeeds(plants)),
        uprootSeed: (plotId: number) => dispatch(uprootSeed(plotId)),
        increaseInventory: (item: Item) => dispatch(increaseInventory(item)),
        addHistoryData: (category: string, name: string, amount: number) => dispatch(addHistoryData(category, name, amount))
    };
}

const emptylist: () => number[] = () => [];

class ConnectedFarm extends Component<IFarmProps, IFarmState> {
    timeout: any = undefined;
    state = {
        dialogue: Dialogue.None,
        chosenPlots: emptylist(),
        chosenSeed: new Plant(CropName.empty),
    };

    constructor(props: IFarmProps) {
        super(props);
        this.seedClick = this.seedClick.bind(this);
        this.plotClick = this.plotClick.bind(this);
        this.seedIsUnlocked = this.seedIsUnlocked.bind(this);
    }


    componentDidUpdate(prevProps: IFarmProps) {
        //console.log("update", this.state, this.props, prevProps);
    }

    seedClick(plant: Plant) {
        this.setState({
            chosenSeed: plant,
        });
        this.plantSeed(plant)
    }


    choosePlot(id: number) {
        let chosenPlots: number[] = this.state.chosenPlots;

        function check() {
            for (let i = 0; i < chosenPlots.length; i++) {
                if (chosenPlots[i] == id) {
                    chosenPlots.splice(i, 1);
                    return true;
                }
            }
            return false;
        }

        if (!check()) {
            chosenPlots.push(id)
        }
        this.setState({
            chosenPlots: chosenPlots
        });
    }

    plotClick(plant: Plant, finished: boolean, timeLeft: number) {
        if (plant.name == CropName.empty) {
            return this.choosePlot(plant.plotId);
        }
        if (finished) {
            return this.harvest(plant)
        }
        this.showUprootSeedDialogue(plant, timeLeft);
    }

    harvest(plant: Plant) {
        let crop = plant.produce();
        Fetch.put("farmPlots/harvest", {
            plant: "harvest",
            plots: [plant.plotId]
        })
            .then(a => a.json())
            .then((result) => {
                console.log("success", result);
                crop.amount = result[crop.name];
                this.props.increaseInventory(crop);
                this.props.addHistoryData("plants", crop.name, crop.amount)
            })
            .catch(e => {
                console.log("failed", e);
            });
        this.props.harvestSeeds([plant]);
    }

    showUprootSeedDialogue(plant: Plant, timeLeft: number) {
        this.timeout = setTimeout(() => {
            if (this.state.dialogue === Dialogue.Uproot) {
                this.setState({
                    dialogue: Dialogue.None
                })
            }
        }, (timeLeft - 0.5) * 1000);
        this.setState({
            dialogue: Dialogue.Uproot,
            chosenPlots: [plant.plotId],
        })
    }

    uprootSeed() {
        this.props.uprootSeed(this.state.chosenPlots[0]);
        Fetch.put("farmPlots/harvest", {
            plant: "remove",
            plots: this.state.chosenPlots
        }).catch((error) => {
            console.warn(error);
        })
    }

    plantSeed(chosenSeed: Plant) {
        let plants: Plant[] = [];
        let chosenPlots = this.state.chosenPlots;
        if (chosenPlots.length == 0) {
            for (let key in this.props.farmPlots) {
                if (this.props.farmPlots[key].name == "empty") {
                    chosenPlots.push(this.props.farmPlots[key].plotId);
                }
            }
        }
        for (let i in chosenPlots) {
            //console.log(i,this.state.chosenPlots[i]);
            plants.push(new Plant(
                chosenSeed.name,
                this.state.chosenPlots[i])
            );
        }
        if (chosenPlots.length == 0) {
            return
        }
        Fetch.put("farmPlots/plant", {
            plant: chosenSeed.name,
            plots: this.state.chosenPlots
        }).catch((error) => {
            console.warn(error);
        });
        this.props.plantSeeds(plants);
        this.unselectAll();
    }

    unselectAll() {
        this.setState({
            chosenPlots: []
        })
    }


    showDialogue() {
        switch (this.state.dialogue) {
            case Dialogue.None:
                return null;
            case Dialogue.Plant:
                return this.plantDialogue();
            case Dialogue.Uproot:
                return this.uprootSeedDialogue();
        }
    }

    closeDialogue() {
        this.setState({
            dialogue: Dialogue.None,
            chosenPlots: []
        });
        if (this.timeout) {
            clearTimeout(this.timeout);
            this.timeout = undefined;
        }
    }

    plantDialogue() {
        return DialogueStandards.YES_NO_DIALOGUE
        (
            "Planting",
            `Do you want to plant ${Strings.decamelizeLower(this.state.chosenSeed.name)}?`,
            () => {
                this.plantSeed(this.state.chosenSeed);
                this.closeDialogue()
            },
            () => this.closeDialogue()
        )
    }

    uprootSeedDialogue() {
        return DialogueStandards.YES_NO_DIALOGUE
        (
            "Uprooting",
            `Are you sure you want to uproot this ${Strings.decamelizeLower(this.props.farmPlots[this.state.chosenPlots[0]].name)}?`,
            () => {
                this.uprootSeed();
                this.closeDialogue()
            },
            () => {
                this.closeDialogue()
            }
        )
    }

    seedIsUnlocked(seed: CropName): boolean{
        for(let key in this.props.unlockedSeeds){
            if(this.props.unlockedSeeds[key].name === seed){
                return true;
            }
        }
        return false;
    }

    showMill() {
        let name = "mill";
        let processDataList =
            {
                wheatFlour:
                    {
                        input: [new Crop(CropName.wheat, 20)],
                        output: [new Food(FoodName.flour, 1)],
                        time: 30,
                        show: ()=>true
                    },
                barleyBarleyFlour:
                    {
                        input: [new Crop(CropName.barley, 50)],
                        output: [new Food(FoodName.barleyFlour, 1)],
                        time: 60,
                        show: ()=>this.seedIsUnlocked(CropName.barley)
                    },

            };
        return ProcessingPlantFactory.create(name, processDataList);
    }

    showBaker() {
        let name = "baker";
        let processDataList =
            {
                flourBread:
                    {
                        input: [new Food(FoodName.flour, 30)],
                        output: [new Food(FoodName.bread, 1)],
                        time: 180,
                        show: ()=>true
                    },
                barleyFlourWholeGrainBread:
                    {
                        input: [new Food(FoodName.barleyFlour, 20)],
                        output: [new Food(FoodName.wholeGrainBread, 1)],
                        time: 300,
                        show: ()=>this.seedIsUnlocked(CropName.barley)
                    },
                floursMultiGrainBread:
                    {
                        input: [
                            new Food(FoodName.barleyFlour, 10),
                            new Food(FoodName.flour, 10),
                        ],
                        output: [new Food(FoodName.multiGrainBread, 1)],
                        time: 360,
                        show: ()=>this.seedIsUnlocked(CropName.barley)
                    },
            };
        return ProcessingPlantFactory.create(name, processDataList);
    }

    render = () => (
        <div className="Farm">
            {this.showDialogue()}
            <div className={"farmplots"}>
                {this.props.farmPlots.map(
                    (plant: Plant, i: number) =>
                        <FarmingPlot
                            plant={plant}
                            key={plant.plotId}
                            chosen={this.state.chosenPlots.includes(plant.plotId)}
                            harvest={this.plotClick}
                        />)
                }
            </div>
            <div className="seeds">
                {this.props.unlockedSeeds.map((a, i) => <Seed plant={a} key={i} plantSeeds={this.seedClick}/>)}
            </div>
            <div className={"processors"}>
                {this.showMill()}
                {this.showBaker()}
            </div>

        </div>
    )
}

const Farm = connect(mapStateToProps, mapDispatchToProps)(ConnectedFarm);

export default Farm;