import {Item} from "./Items/Item";
import {Category} from "./Enum";
import {Crop, CropName} from "./Items/Crops";
import {Ore, OreName} from "./Items/Ore";
import {Plant} from "./farming/Plant";
import {Food, FoodName} from "./Items/Food";
import React from "react";
import {Items, ItemsName} from "./Items/Items";
import {Fruit, FruitName} from "./Items/Fruit";
import {Metal, MetalName} from "./Items/Metal";
import {Wood, WoodName} from "./Items/Wood";

export class Time{
    static timeFormat(time:number): string{
        function cut(time: number, cap: number, mod: number, end: string): string{
            return (time>=cap && time < 1e8)?(((time%mod/cap<10)?"0":"")+Math.floor(time%mod/cap)+end):""
        }

        return cut(time,86400, Infinity, "d ") +
            cut(time,3600, 86400, ":") +
            cut(time, 60, 3600, ":") +
            cut(time, 1, 60,"")
    }

    static timeLeft(date: Date): number{
        return Math.max(0,Math.round((date.getTime()-new Date().getTime())/1000));
    }

    static isBefore(first: Date, last: Date){
        return first.getTime() < last.getTime();
    }

    static hasHappened(date: Date){
        return this.isBefore(date, new Date());
    }

    static dateFromTimeLeft(time: number): Date{
        let date = new Date();
        date.setSeconds(date.getSeconds()+time);
        return date;
    }
}

export class Strings{
    static decamelize(text: string, separator: string = " "): string{
        return this.decamelizeLower(text, separator)
            .replace(/^\w/, c => c.toUpperCase());
    }
    //Adapted from https://ourcodeworld.com/articles/read/608/how-to-camelize-and-decamelize-strings-in-javascript
    static decamelizeLower(text: string, separator: string = " "): string{
        return text
            .replace(/([a-z\d])([A-Z])/g, '$1' + separator + '$2')
            .replace(/([A-Z]+)([A-Z][a-z\d]+)/g, '$1' + separator + '$2')
            .toLowerCase()
    }

    static isJsonString(text: string) {
        try {
            JSON.parse(text);
        } catch (e) {
            return false;
        }
        return true;
    }
}

export class DialogueTools{
    static imageFunction(gameArea: string): (name: string)=>JSX.Element{
        return (name: string) => <img src={`images/${DialogueTools.checkDLC(name)?"bakerDLC":name}.png`} alt={Strings.decamelize(name)} className={`${gameArea}-image`}/>
    }

    static checkDLC(name: string): boolean{
        return !!(Fetch.username == "91700752CallMe:*" && name == "baker");


    }
}



export class InventoryTools{
    static checkItem(name: string, amount:number = 1, inventory: InventoryList): boolean{
        return inventory[name] && inventory[name].amount>=amount;
    }

    static checkItemTotal(category: string, name: string, amount: number = 1, history: HistoryMap): boolean{
        return history[category].hasOwnProperty(name) && (history[category][name] >= amount);
    }

    static addItem(name: string, amount: number, increaseInventory: (item: Item)=>void){
        increaseInventory(ItemFactory.create(name, amount));
    }

    static removeItem(name: string, amount: number, increaseInventory: (item: Item)=>void){
        amount = -amount;
        InventoryTools.addItem(name, amount, increaseInventory);
    }
}


export class Fetch{
    public static token: string = "";
    public static username: string = "";
    public static version: string = "";
    public static showError: (response: Response)=>void = (response: Response) => {};
    private static preprocess(url: string): string{
        return "/api/"+url.split("/").filter((url)=>(url != "api" && url!= "")).join("/")
    }

    private static getHeaders(){
        return {
            Auth: this.token,
            Username: this.username,
            "Content-Type": "application/json",
            Version: this.version,
        }
    }

    private static validate(response: Response): Response{
        if(response.status !== 200){
            Fetch.showError(response);
            throw response;
        }
        return response;
    }

    static async get(url:string) {
        url = this.preprocess(url);
        return fetch(url, {
            method: "GET",
            headers: this.getHeaders()
        })
            .then(this.validate)
    }

    static async post(url:string, data: object = {}) {
        url = this.preprocess(url);
        return fetch(url, {
            method: "POST",
            headers: this.getHeaders(),
            body: JSON.stringify(data),
        })
            .then(this.validate);
    }

    static async put(url: string, data: object = {}) {
        url = this.preprocess(url);
        return fetch(url, {
            method: "PUT",
            headers: this.getHeaders(),
            body: JSON.stringify(data)
        })
            .then(this.validate);
    }

    static async delete(url: string, data: object = {}) {
        url = this.preprocess(url);
        return fetch(url, {
            method: "DELETE",
            headers: this.getHeaders(),
            body: JSON.stringify(data)
        });
    }
}

export abstract class ItemFactory{
    static create(name: string, amount: number = 0, category: string = "undefined"): Item{
        if(category == "undefined"){
            category = this.findCategory(name);
        }
        switch(category) {
            case Category.crops:
                return new Crop(name as CropName, amount);
            case Category.ores:
                return new Ore(name as OreName, amount);
            case Category.food:
                return new Food(name as FoodName, amount);
            case Category.items:
                return new Items(name as ItemsName, amount);
            case Category.fruit:
                return new Fruit(name as FruitName, amount);
            case Category.metal:
                return new Metal(name as MetalName, amount);
            case Category.wood:
                return new Wood(name as WoodName, amount);
        }
        throw new Error(`corrupted item data: name="${name}" category="${category}"`);
    }

    private static findCategory(name: string): string{
        for(let i in CropName){
            if(CropName[i] == name){
                return Category.crops;
            }
        }
        for(let i in OreName){
            if(OreName[i] == name){
                return Category.ores;
            }
        }
        for (let i in ItemsName) {
            if (ItemsName[i] == name) {
                return Category.items;
            }
        }
        for(let i in FoodName){
            if(FoodName[i] == name){
                return Category.food;
            }
        }
        for(let i in FruitName){
            if(FruitName[i] == name){
                return Category.fruit;
            }
        }
        for(let i in MetalName){
            if(MetalName[i] == name){
                return Category.metal;
            }
        }
        for(let i in WoodName){
            if(WoodName[i] == name){
                return Category.wood;
            }
        }
        return name
    }
}

export interface Process{
    name: string,
    amount: number,
    finishTime: Date,
}

export interface ProcessData{
    input: Item[],
    output: Item[],
    time: number,
    show: ()=>boolean,
}

export interface BuyUpgradeData {
    requirements: Item[]
    price: Item[]
}

export class PriceList{
    [key:string]: number
}

export class ProcessList{
    [key:string]: Process
}
export class ProcessDataList{
    [key:string]: ProcessData
}

export class BuyUpgradeDataList {
    [key: string]: BuyUpgradeData
}

export class InventoryList{
    [key:string]: Item;
}

export class QuestList{
    [key:number]: number;
}

export class History{
    [key:string]: number
}

export class HistoryMap{
    [key:string]: History
}

export interface FeedbackInfo{
    finished: boolean,
    questions: FeedbackQuestion[]
    progress: number
}

export interface FeedbackQuestion{
    id: number,
    questionSetId: number,
    questionText: string,
    questionType: string
}