import React, {Component} from 'react';
import './Forest.css';
import {addHistoryData, chopTree, harvestFruit, increaseInventory, plantTrees} from "../actions/actions";
import {connect} from "react-redux";
import {DialogueStandards} from "../dialogue/DialogueStandards";
import {ProcessingPlantFactory} from "../factories/ProcessingFactory";
import {Fetch, InventoryList, ItemFactory, Time} from "../utils";
import {Tree, TreeName} from "./Tree";
import {ReduxState} from "../reducers/reducer";
import {Item} from "../Items/Item";
import {Fruit} from "../Items/Fruit";
import TreePatch from "./TreePatch";
import {FruitSeed} from "./FruitSeed";
import {Wood, WoodName} from "../Items/Wood";
import {Category} from "../Enum";

enum Dialogue {
    None,
    Plant,
    Chop
}

interface IForestState {
    dialogue: Dialogue,
    chosenPatches: number[],
    chosenTree: Tree
}

interface IForestProps {
    treePatches: Tree[],
    inventory: InventoryList,
    harvestFruit: (patches: number[]) => void,
    plantTrees: (tree: Tree[]) => void,
    chopTrees: (patches: number[]) => void,
    increaseInventory: (item: Item) => void,
    addHistoryData: (category: string, name: string, amount: number) => void,
}

const mapStateToProps = (state: ReduxState) => {
    return {
        treePatches: state.treePatches,
        inventory: state.inventory
    }
};

function mapDispatchToProps(dispatch: any) {
    return {
        plantTrees: (trees: Tree[]) => dispatch(plantTrees(trees)),
        harvestFruit: (patches: number[]) => dispatch(harvestFruit(patches)),
        chopTrees: (patches: number[]) => dispatch(chopTree(patches)),
        increaseInventory: (item: Item) => dispatch(increaseInventory(item)),
        addHistoryData: (category: string, name: string, amount: number) => dispatch(addHistoryData(category, name, amount))
    };
}

const emptylist: () => number[] = () => [];

class ConnectedForest extends Component<IForestProps, IForestState> {
    timeout: any = undefined;
    state = {
        dialogue: Dialogue.None,
        chosenTree: new Tree(TreeName.empty),
        chosenPatches: emptylist()
    };

    constructor(props: IForestProps) {
        super(props);
        this.plotClick = this.plotClick.bind(this);
        this.seedClick = this.seedClick.bind(this);
        this.harvestAllFruit = this.harvestAllFruit.bind(this);
    }


    componentDidUpdate(prevProps: IForestProps) {
        //console.log("update", this.state, this.props, prevProps);
    }


    choosePlot(id: number) {
        let chosenPatches: number[] = this.state.chosenPatches;

        function check() {
            for (let i = 0; i < chosenPatches.length; i++) {
                if (chosenPatches[i] == id) {
                    chosenPatches.splice(i, 1);
                    return true;
                }
            }
            return false;
        }

        if (!check()) {
            chosenPatches.push(id)
        }
        this.setState({
            chosenPatches: chosenPatches
        });
    }

    plotClick(tree: Tree, finished: boolean, finishedFruit: boolean) {
        if (tree.name == TreeName.empty) {
            return this.choosePlot(tree.patchId);
        }
        if (finishedFruit) {
            return this.harvestFruit(tree)
        }
        if(finished){
            return this.choosePlot(tree.patchId);
        }
    }

    seedClick(fruit: Fruit) {
        this.plantTree(new Tree(fruit.name + "Tree" as TreeName));
        this.closeDialogue()
    }


    harvestFruit(tree: Tree) {
        let fruit = tree.harvestFruit();
        Fetch.put("treePatches/harvestFruit", {
            plots: [tree.patchId]
        })
            .then(a => a.json())
            .then((result) => {
                fruit.amount = result[fruit.name];
                this.props.increaseInventory(fruit);
                this.props.addHistoryData("trees",fruit.name, fruit.amount)
            })
            .catch(e => {
                console.log("failed", e);
            });
        this.props.harvestFruit([tree.patchId]);
    }

    harvestAllFruit() {
        let chosenPatches = [];
        for (let key in this.props.treePatches) {
            if (Time.hasHappened(this.props.treePatches[key].finishTimeFruit) && this.props.treePatches[key].name != TreeName.empty) {
                chosenPatches.push(this.props.treePatches[key].patchId);
            }
        }

        if(chosenPatches.length==0){
            return;
        }

        Fetch.put("treePatches/harvestFruit", {
            plots: chosenPatches
        })
            .then(a => a.json())
            .then((result) => {
                for (let property in result) {
                    let item = ItemFactory.create(property, result[property]);
                    if(item.category == Category.fruit){
                        this.props.addHistoryData("trees",item.name, item.amount);
                    }
                    this.props.increaseInventory(item);
                }
            })
            .catch(e => {
                console.log("failed", e);
            });
        this.unselectAll()
        this.props.harvestFruit(chosenPatches);
    }

    chopTree() {
        let chosenPatches = this.state.chosenPatches;
        if (chosenPatches.length == 0) {
            for (let key in this.props.treePatches) {
                if (Time.hasHappened(this.props.treePatches[key].finishTime) && this.props.treePatches[key].name != TreeName.empty) {
                    chosenPatches.push(this.props.treePatches[key].patchId);
                }
            }
        }
        if(chosenPatches.length==0){
            return;
        }
        Fetch.put("treePatches/chopTree", {
            plots: chosenPatches
        })
            .then(a => a.json())
            .then((result) => {
                for (let property in result) {
                    let item = ItemFactory.create(property, result[property]);
                    if(item.category == Category.fruit){
                        this.props.addHistoryData("trees",item.name, item.amount);
                    }
                    this.props.increaseInventory(item);
                }


            })
            .catch(e => {
                console.log("failed", e);
            });
        this.props.chopTrees(chosenPatches);
    }

    plantTree(chosenTree: Tree) {
        let trees: Tree[] = [];
        let chosenPatches = this.state.chosenPatches;
        if (chosenPatches.length == 0) {
            for (let key in this.props.treePatches) {
                if (this.props.treePatches[key].name == "empty") {
                    chosenPatches.push(this.props.treePatches[key].patchId);
                }
            }
        }
        chosenPatches = chosenPatches.slice(0, this.props.inventory[chosenTree.harvestFruit().name].amount);
        let realPatches = [];
        for (let i in chosenPatches) {
            if(this.props.treePatches[chosenPatches[i]].name != TreeName.empty){
                continue;
            }
            trees.push(new Tree(
                chosenTree.name,
                this.state.chosenPatches[i])
            );
            realPatches.push(chosenPatches[i])
        }
        if (realPatches.length == 0) {
            return
        }

        Fetch.put("treePatches/plant", {
            plant: chosenTree.name,
            plots: realPatches
        }).catch((error) => {
            console.warn(error);
        });
        let fruit = chosenTree.harvestFruit();
        fruit.amount = -trees.length;
        this.props.increaseInventory(fruit);
        this.props.plantTrees(trees);
        this.unselectAll();
    }

    unselectAll() {
        this.setState({
            chosenPatches: []
        })
    }


    showDialogue() {
        switch (this.state.dialogue) {
            case Dialogue.None:
                return null;
            case Dialogue.Chop:
                if (this.state.chosenPatches.length > 0) {
                    return this.chopTreeDialogue();
                } else {
                    return this.chopAllTreesDialogue();
                }
            case Dialogue.Plant:
                return this.plantTreeDialogue()
        }
    }

    closeDialogue() {
        this.setState({
            dialogue: Dialogue.None,
            chosenPatches: []
        });
        if (this.timeout) {
            clearTimeout(this.timeout);
            this.timeout = undefined;
        }
    }

    chopTreeDialogue() {
        return DialogueStandards.YES_NO_DIALOGUE
        (
            `Chop tree${this.state.chosenPatches.length > 1 ? "s" : ""}`,
            `Are you sure you want to chop the selected tree${this.state.chosenPatches.length > 1 ? "s" : ""}?`,
            () => {
                this.chopTree();
                this.closeDialogue()
            },
            () => this.closeDialogue()
        )
    }

    plantTreeDialogue() {
        return DialogueStandards.CANCEL_DIALOGUE_ELEMENT_BODY
        (
            "Plant tree",
            <div className="fruit">
                <span className="fruit-seed-title"> Choose what type of tree you want to plant</span>
                {Object.keys(this.props.inventory)
                    .map(key => {return this.props.inventory[key]})
                    .filter((item: Item) => item.category == "fruit")
                    .filter((item: Item) => item.amount > 0)
                    .map((a, i) => <FruitSeed fruit={a} key={i} plantFruitSeeds={this.seedClick}/>)}
            </div>,
            () => {
                this.closeDialogue()
            }
        )
    }

    chopAllTreesDialogue() {
        return DialogueStandards.YES_NO_DIALOGUE
        (
            "Chop all trees",
            `Are you sure you want to chop all fully grown trees?`,
            () => {
                this.chopTree();
                this.closeDialogue()
            },
            () => this.closeDialogue()
        )
    }

    showWoodWorker() {
        let name = "woodWorker";
        let processDataList =
            {
                logsPlanks:
                    {
                        input: [new Wood(WoodName.logs, 1)],
                        output: [new Wood(WoodName.planks, 2)],
                        time: 900,
                        show: ()=>true
                    }
            };
        return ProcessingPlantFactory.create(name, processDataList);
    }

    render = () => (
        <div className="Forest">
            {this.showDialogue()}
            <div className={"trePatches"}>
                {this.props.treePatches.map(
                    (tree: Tree, i: number) =>
                        <TreePatch
                            tree={tree}
                            key={tree.patchId}
                            chosen={this.state.chosenPatches.includes(tree.patchId)}
                            plotClick={this.plotClick}
                        />)
                }
            </div>
            <div className="tree-actions">
                <img src={"images/trowel.png"} alt="Plant Tree"
                     onClick={() => this.setState({dialogue: Dialogue.Plant})}/>
                <img src={"images/axe.png"} alt="Chop Tree" onClick={() => this.setState({dialogue: Dialogue.Chop})}/>
                <img src={"images/basket.png"} alt="Harvest Fruit" onClick={() => this.harvestAllFruit()}/>
            </div>

            <div className={"processors"}>
                <div className={"processors"}>
                    {this.showWoodWorker()}
                </div>
            </div>
        </div>
    )
}

const Forest = connect(mapStateToProps, mapDispatchToProps)(ConnectedForest);

export default Forest;